import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames/bind';
import AnueSearchInput from '../AnueSearchInput/AnueSearchInput';
import AnueSearchRealtimeResult from '../AnueSearchRealtimeResult/AnueSearchRealtimeResult';
import AnueSearchFetch from './AnueSearchFetch';
import SearchResultHelper from './helpers';
import debounce from '../../utils/debounce';
import styles from './AnueSearch.scss';
import AnueLink from '../../utils/anueLink';
import AnueCookie from '../../utils/anueCookie';

const cx = classnames.bind(styles);

export default class AnueSearch extends Component {
  static propTypes = {
    maxRecentHistory: PropTypes.number,
    theme: PropTypes.string, // mobile | desktop
    // AnueSearchInputProps
    customAttribute: PropTypes.objectOf(PropTypes.any),
    customWrapperStyles: PropTypes.objectOf(PropTypes.any),
    customInputWrapperStyles: PropTypes.objectOf(PropTypes.any),
    customResultWrapperStyles: PropTypes.objectOf(PropTypes.any),
    customInputStyles: PropTypes.objectOf(PropTypes.any),
    customIconStyles: PropTypes.objectOf(PropTypes.any),
    shouldAlwaysDisplayInput: PropTypes.bool,
    onIconClick: PropTypes.func,
    onInputChange: PropTypes.func,
    onInputFocus: PropTypes.func,
    onInputBlur: PropTypes.func,
    onInputPressEnter: PropTypes.func,
    onSearchVisibilityChange: PropTypes.func,
    placeholder: PropTypes.string,
    apiEnv: PropTypes.string,
    aink: PropTypes.string,
  };
  static defaultProps = {
    maxRecentHistory: 5,
    theme: null,
    customAttribute: null,
    customWrapperStyles: null,
    customInputWrapperStyles: null,
    customResultWrapperStyles: null,
    customInputStyles: null,
    customIconStyles: null,
    customInputClassName: null,
    customResultClassName: null,
    shouldAlwaysDisplayInput: false,
    placeholder: '搜尋新聞、代碼或名稱',
    onIconClick: () => {},
    onInputChange: () => {},
    onInputFocus: () => {},
    onInputBlur: () => {},
    onInputPressEnter: () => {},
    onSearchVisibilityChange: () => {},
    apiEnv: 'prod',
    aink: null,
  };
  constructor(props) {
    super(props);
    this.state = {
      isInputHidden: false,
      isInputFocus: false,
      hotTags: null,
      hotTrades: null,
      recentSearch: null,
      hotDrivers: null,
      searchString: '',
      tradeResult: null,
      newsResult: null,
      driversResult: null,
      hover: false,
    };

    this.request = new AnueSearchFetch();
    this.anueUrlSingleton = new AnueLink(props.apiEnv);
    this.anueCookieSingleton = new AnueCookie(props.apiEnv);
    this.dataHelpers = SearchResultHelper(this.anueUrlSingleton);
  }

  componentDidMount() {
    if (typeof window !== 'undefined') {
      const keywordQueryStringRegex = location.search && decodeURIComponent(location.search).match(/keyword=([^&]*)/);

      if (keywordQueryStringRegex && keywordQueryStringRegex[1]) {
        const keyword = keywordQueryStringRegex[1];

        this.inputRef.value = keyword;
      }
      // The clicked event of AnueSearch is set to stopPropagation.
      // So click the anywhere outside of AnueSearch will hide the AnueRealtimeResult.
      window.addEventListener('click', this.handleHideSearchResult);
    }
  }

  componentDidUpdate(prevPrev, prevState) {
    const { hover } = this.state;

    if (typeof window !== 'undefined' && prevState.hover !== hover) {
      const bodyStyles = document.body.style;

      if (hover) {
        document.body.style.overflowY = 'hidden';
      } else if (bodyStyles) {
        document.body.style.overflowY = 'auto';
      }
    }
  }

  componentWillUnmount() {
    window.removeEventListener('click', this.handleHideSearchResult);
  }

  getSearchHistory = () => {
    const history = this.anueCookieSingleton.getSearchCookies();

    this.setState({
      recentSearch: this.dataHelpers.recentSearchDataHelper(history),
    });
  };

  storeSearchHistory = (string, shouldDelete) => {
    const { maxRecentHistory, apiEnv } = this.props;

    this.anueCookieSingleton.setSearchCookies({ string, shouldDelete, cookieLimit: maxRecentHistory, env: apiEnv });
    this.getSearchHistory();
  };

  removeAllTags = () => {
    const { apiEnv } = this.props;

    this.anueCookieSingleton.removeSearchCookies({ env: apiEnv });
    this.getSearchHistory();
  };

  handleClickRecentTag = tagName => {
    const { aink } = this.props;

    const link = this.anueUrlSingleton.generateSearchResultLink({
      keyword: tagName,
      backLink: aink,
    });

    location.href = link;
  };

  handleClickRemoveTag = tagName => {
    this.storeSearchHistory(tagName, true);
  };

  handleFetch = debounce(async () => {
    const { searchString } = this.state;

    const rawResult = await this.request.get(`${this.anueUrlSingleton.anueSearchAllApiUrl}${searchString}`);
    const result = this.dataHelpers.searchTradesResponseHelper(rawResult);

    if (result && (result.quoteFunds || result.news || result.oldDrivers)) {
      this.storeSearchHistory(searchString);
      this.setState({
        tradeResult: result.quoteFunds,
        newsResult: result.news,
        driversResult: result.oldDrivers,
      });
    }
  }, 500);

  handleSearchChange = value => {
    const { onInputChange } = this.props;

    this.setState({ searchString: value });

    onInputChange(value);

    this.handleFetch();
  };

  handlePressEnter = () => {
    const { onInputPressEnter } = this.props;
    const { searchString } = this.state;

    onInputPressEnter();
    if (searchString) this.storeSearchHistory(searchString);
  };

  handleClickSearchIcon = () => {
    const { onIconClick, aink } = this.props;
    const { searchString, hover } = this.state;

    onIconClick();
    if (!!searchString && typeof window !== 'undefined') {
      this.storeSearchHistory(searchString);
      const link = this.anueUrlSingleton.generateSearchResultLink({ keyword: searchString, backLink: aink });

      location.href = link;
    }
    if (hover) {
      this.setState({
        hover: false,
      });
    }
  };

  handleSearchOnFocus = async () => {
    const { onInputFocus } = this.props;
    const { hotTags, hotTrades, hotDrivers } = this.state;

    this.setState({ hover: true });
    onInputFocus();

    if (!hotTags && !hotTrades && !hotDrivers) {
      const _hotTrades = await this.request.get(this.anueUrlSingleton.hotTradesApiUrl);
      const _hotTags = await this.request.get(this.anueUrlSingleton.hotTagsApiUrl);
      const _hotDrivers = await this.request.get(this.anueUrlSingleton.hotDriversApiUrl);

      this.getSearchHistory();
      this.setState({
        hotTags: this.dataHelpers.hotTagsResponseHelper(_hotTags),
        hotTrades: this.dataHelpers.hotTradesResponseHelper(_hotTrades),
        hotDrivers: this.dataHelpers.hotDriversResponseHelper(_hotDrivers),
      });
    }
  };

  handleHideSearchResult = () => {
    const { hover } = this.state;

    if (hover) {
      this.setState({
        hover: false,
      });
    }
  };

  handleSearchOnBlur = () => {
    const { onInputBlur } = this.props;

    onInputBlur();
  };

  handleStopWrapperClickPropagation = e => {
    e.stopPropagation();
  };

  render() {
    const {
      theme,
      shouldAlwaysDisplayInput,
      placeholder,
      customAttribute,
      customWrapperStyles,
      customInputWrapperStyles,
      customResultWrapperStyles,
      customInputStyles,
      customIconStyles,
      onSearchVisibilityChange,
      aink,
    } = this.props;
    const {
      hotTags,
      hotTrades,
      hotDrivers,
      recentSearch,
      searchString,
      hover,
      tradeResult,
      newsResult,
      driversResult,
    } = this.state;
    const searchResultLink = this.anueUrlSingleton.generateSearchResultLink({ keyword: searchString, backLink: aink });

    return (
      <div
        className={cx('anue-search', theme)}
        style={customWrapperStyles}
        onClick={this.handleStopWrapperClickPropagation}
      >
        <div className={cx('input-wrapper', { active: hover })}>
          <AnueSearchInput
            theme={theme}
            placeholder={placeholder}
            onInputChange={this.handleSearchChange}
            onInputFocus={this.handleSearchOnFocus}
            onInputBlur={this.handleSearchOnBlur}
            onInputPressEnter={this.handlePressEnter}
            onIconClick={this.handleClickSearchIcon}
            shouldAlwaysDisplayInput={shouldAlwaysDisplayInput || !!searchString}
            value={searchString}
            customWrapperStyles={customInputWrapperStyles}
            customInputStyles={customInputStyles}
            customIconStyles={customIconStyles}
            onDisplayChange={onSearchVisibilityChange}
            customAttribute={{
              ...customAttribute,
              ref: input => {
                this.inputRef = input;
              },
            }}
          />
        </div>
        <div className={cx('search-result', theme, { display: hover })}>
          <AnueSearchRealtimeResult
            hotTags={hotTags}
            hotTrades={hotTrades}
            hotDrivers={hotDrivers}
            recentSearch={recentSearch}
            searchString={searchString}
            tradeResult={tradeResult}
            newsResult={newsResult}
            driversResult={driversResult}
            onRecentSearchRowClick={this.handleClickRecentTag}
            onRecentSearchRowRemoveClick={this.handleClickRemoveTag}
            onRecentSearchRemoveAllClick={this.removeAllTags}
            customWrapperStyles={customResultWrapperStyles}
            searchResultPageLink={searchResultLink}
          />
        </div>
      </div>
    );
  }
}
