import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames/bind';
import SearchResultContentWrapper from './SearchResultContentWrapper';
import { numSignHelper } from '../../utils/num';
import {
  hotTagsType,
  hotTradesType,
  hotDriversType,
  recentSearchType,
  tradeResultType,
  newsResultType,
} from '../../utils/propTypes';
import oldDriverIcon from '../../assets/icon-default-driver.png';
import styles from './AnueSearchRealtimeResult.scss';
import LazyImage from '../LazyImage/LazyImage';
import AnueLink from '../../utils/anueLink';
import { driverNameEllipsisHelper, newsPublishedDateHelper } from './helpers';

const cx = classnames.bind(styles);
const anueUtilLink = new AnueLink('prod');

function AnueSearchRealtimeResult({
  hotTags,
  hotTrades,
  hotDrivers,
  recentSearch,
  tradeResult,
  newsResult,
  driversResult,
  searchString,
  customWrapperStyles,
  onRecentSearchRowClick,
  onRecentSearchRowRemoveClick,
  onRecentSearchRemoveAllClick,
  searchResultPageLink,
}) {
  const defaultSearchPageLink = anueUtilLink.generateSearchResultLink({
    keyword: searchString,
  });
  const searchPageLink = searchResultPageLink || defaultSearchPageLink;
  const renderHotTags = () => (
    // Display nothing if receive a empty array
    <SearchResultContentWrapper isLoaded={hotTags && hotTags.length > 0}>
      <div className={cx('hot-tags')}>
        <div>
          <p>熱搜：</p>
        </div>
        <div>
          {hotTags &&
            hotTags.map(tag => (
              <a key={tag.tagName} href={tag.link}>
                {tag.tagName}
              </a>
            ))}
        </div>
      </div>
    </SearchResultContentWrapper>
  );
  const renderHotTrades = () => (
    // Display nothing if receive a empty array
    <SearchResultContentWrapper title="熱門行情" isLoaded={hotTrades && hotTrades.length > 0}>
      <div className={cx('content-rows')}>
        {hotTrades &&
          hotTrades.map((trade, i) => (
            // eslint-disable-next-line react/no-array-index-key
            <a key={`hot-trade${i}`} className={cx('row', 'icon--hot')} href={trade.link}>
              {trade.localName}
            </a>
          ))}
      </div>
    </SearchResultContentWrapper>
  );
  const renderRecentSearch = () => (
    // Display nothing if receive a empty array
    <SearchResultContentWrapper
      title="最近搜尋"
      headerActionName="全部刪除"
      isLoaded={recentSearch && recentSearch.length > 0}
      onHeaderActionClick={onRecentSearchRemoveAllClick}
    >
      <div className={cx('content-rows')}>
        {recentSearch &&
          recentSearch.map((search, i) => {
            const handleClickRow = () => onRecentSearchRowClick(search.keyword);
            const handleClickRemove = () => onRecentSearchRowRemoveClick(search.keyword);

            return (
              // eslint-disable-next-line react/no-array-index-key
              <div key={`recent-search${i}`} className={cx('row', 'limit-length', 'icon--time')}>
                <a onClick={handleClickRow}>{search.keyword}</a>
                <p className={cx('icon-cross')} onClick={handleClickRemove} />
              </div>
            );
          })}
      </div>
    </SearchResultContentWrapper>
  );
  const renderDrivers = (driverList, isSearchResult) => (
    // isSearchResult ? Display the default message if receive a empty array : // Display nothing if receive a empty array
    <SearchResultContentWrapper
      title="基金老司機"
      isLoaded={isSearchResult ? driverList : driverList && driverList.length > 0}
      isCols
    >
      <div className={cx('content-cols')}>
        {driverList && driverList.length > 0 ? (
          driverList.map((driver, i) => (
            // eslint-disable-next-line react/no-array-index-key
            <a key={`driver${i}`} className={cx('col')} href={driver.link}>
              <div className={cx('avatar')}>
                <LazyImage isRounded src={driver.avatarPicture || oldDriverIcon} />
              </div>
              <p
                className={cx('mark', 'name')}
                dangerouslySetInnerHTML={{ __html: driverNameEllipsisHelper(driver.driverName) }}
              />
              <span className={cx('sub-title')}>一年績效</span>
              <span className={cx('perf', { red: driver.return1year > 0, green: driver.return1year < 0 })}>
                {typeof driver.return1year !== 'undefined' ? numSignHelper(driver.return1year.toFixed(2)) : '----'}
              </span>
            </a>
          ))
        ) : (
          <div className={cx('empty')}>沒有符合搜尋條件的資料</div>
        )}
      </div>
    </SearchResultContentWrapper>
  );
  const renderSearchTradeResult = () => (
    // Display the default message if receive a empty array.
    <SearchResultContentWrapper title="行情" isLoaded={tradeResult}>
      <div className={cx('content-rows')}>
        {tradeResult && tradeResult.length > 0 ? (
          tradeResult.map((trade, i) => (
            // eslint-disable-next-line react/no-array-index-key
            <a key={`trade${i}`} className={cx('row', 'result')} href={trade.link}>
              <p className={cx('title')}>
                <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: trade.code }} />
                <small>{trade.exchange}</small>
              </p>
              <p className={cx('summary')}>
                <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: trade.chName }} />
                <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: trade.enName }} />
              </p>
            </a>
          ))
        ) : (
          <div className={cx('empty')}>沒有符合搜尋條件的資料</div>
        )}
      </div>
    </SearchResultContentWrapper>
  );
  const renderSearchNewsResult = () => (
    // Display the default message if receive a empty array.
    <SearchResultContentWrapper title="新聞" isLoaded={newsResult}>
      <div className={cx('content-rows')}>
        {newsResult && newsResult.length > 0 ? (
          newsResult.map((n, i) => (
            // eslint-disable-next-line react/no-array-index-key
            <a key={`news${i}`} className={cx('row', 'result')} href={n.link}>
              <p className={cx('title')}>
                <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: n.title }} />
              </p>
              <p className={cx('summary')}>
                <span>{newsPublishedDateHelper(n.publishAt)}</span>
                <span>{n.summary}</span>
              </p>
            </a>
          ))
        ) : (
          <div className={cx('empty')}>沒有符合搜尋條件的資料</div>
        )}
      </div>
    </SearchResultContentWrapper>
  );

  return (
    <div className={cx('anue-search-result')} style={customWrapperStyles}>
      {searchString ? (
        <div className={cx('search-sections', 'section')}>
          <a className={cx('current-search')} href={searchPageLink}>
            查看「{searchString}」的所有搜尋結果＞
          </a>
          {renderSearchTradeResult()}
          {renderSearchNewsResult()}
          {renderDrivers(driversResult, true)}
        </div>
      ) : (
        <div className={cx('hot-sections', 'section')}>
          {renderHotTags()}
          {renderHotTrades()}
          {renderRecentSearch()}
          {renderDrivers(hotDrivers)}
        </div>
      )}
    </div>
  );
}

AnueSearchRealtimeResult.propTypes = {
  hotTags: PropTypes.arrayOf(hotTagsType),
  hotTrades: PropTypes.arrayOf(hotTradesType),
  hotDrivers: PropTypes.arrayOf(hotDriversType),
  recentSearch: PropTypes.arrayOf(recentSearchType),
  tradeResult: PropTypes.arrayOf(tradeResultType),
  newsResult: PropTypes.arrayOf(newsResultType),
  driversResult: PropTypes.arrayOf(hotDriversType),
  searchString: PropTypes.string,
  customWrapperStyles: PropTypes.objectOf(PropTypes.any),
  onRecentSearchRowClick: PropTypes.func,
  onRecentSearchRowRemoveClick: PropTypes.func,
  onRecentSearchRemoveAllClick: PropTypes.func,
  searchResultPageLink: PropTypes.string,
};

AnueSearchRealtimeResult.defaultProps = {
  hotTags: null,
  hotTrades: null,
  hotDrivers: null,
  recentSearch: null,
  tradeResult: null,
  newsResult: null,
  driversResult: null,
  searchTradeResult: null,
  searchDriverResult: null,
  searchNewsResult: null,
  searchString: null,
  customWrapperStyles: null,
  searchResultPageLink: null,
  onRecentSearchRowClick: () => {},
  onRecentSearchRowRemoveClick: () => {},
  onRecentSearchRemoveAllClick: () => {},
};

export default AnueSearchRealtimeResult;
